//
//    rfnoc-hls-neuralnet: Vivado HLS code for neural-net building blocks
//
//    Copyright (C) 2017 EJ Kreinar
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#include <iostream>

#include "fecpredlite0.h"

//hls-fpga-machine-learning insert weights
#include "weights/w2.h"
#include "weights/b2.h"
#include "weights/w4.h"
#include "weights/b4.h"
#include "weights/w6.h"
#include "weights/b6.h"

void fecpredlite0(
	    //input_t input1[N_INPUT_1_1],
	//unsigned short &out_class,
	//result_t layer9_out[N_LAYER_8],
	axis_in *layer1_in,
	axis_out *layerN_out
	//stream_type* layer1_out,
	//stream_type layer1_in[N_INPUT_1_1],
	//stream_type layer1_out[N_INPUT_1_1],
	//stream_type layerN_out[N_LAYER_6]
	//unsigned short &const_size_in_1,
	//unsigned short &const_size_out_1
) {

	//hls-fpga-machine-learning insert IO
	//#pragma HLS INTERFACE ap_ctrl_none port=return
    //#pragma HLS INTERFACE s_axilite port=return bundle=CTRL
	//#pragma HLS INTERFACE s_axilite port=out_class  //bundle=CTRL
	//#pragma HLS INTERFACE s_axilite port=const_size_in_1  bundle=CTRL
	//#pragma HLS INTERFACE s_axilite port=const_size_out_1 bundle=CTRL

	/* These two pragmas define the input and output ports as AXI-Stream (axis) interfaces.
	   An AXI-Stream is FIFO-like, non-addressed communcation bus*/
	#pragma HLS INTERFACE axis depth=13 port=layer1_in  //offset = slave bundle = inDAT//register both
	#pragma HLS INTERFACE axis depth=2 port=layerN_out //offset = slave bundle = outDAT
	//#pragma HLS INTERFACE axis depth=50 port=layer1_in //register both
	//#pragma HLS INTERFACE axis depth=50 port=layer1_out //register both
	//#pragma HLS INTERFACE axis depth=50 port=input1
	//#pragma HLS INTERFACE axis depth=50 port=layer9_out


	#pragma HLS PIPELINE

	//conversion_streamtype tmp_conv;
	uint32 tmp_u32;



    //const_size_in_1 = N_INPUT_1_1;
    //const_size_out_1 = N_LAYER_6;

    // ****************************************
    // NETWORK INSTANTIATION
    // ****************************************

    //hls-fpga-machine-learning insert layers
    input_t input1[N_INPUT_1_1];
    //float input_tmp;
	for(int i=0;i<N_INPUT_1_1;i++){
		float input_tmp;
		input_tmp = (*layer1_in++).data;
		input1[i] = input_tmp;
		//tmp_u32 =layer1_in[i];
		//input_tmp = float(tmp_u32)/65536;
		//tmp_conv.as_uint32 = tmp_u32;
		//input_tmp=tmp_conv.as_float32;
		//input1[i] = input_tmp;
	}

    layer2_t layer2_out[N_LAYER_2];
    #pragma HLS ARRAY_PARTITION variable=layer2_out complete dim=0
    nnet::dense<input_t, layer2_t, config2>(input1, layer2_out, w2, b2);

    layer3_t layer3_out[N_LAYER_2];
    #pragma HLS ARRAY_PARTITION variable=layer3_out complete dim=0
    nnet::relu<layer2_t, layer3_t, relu_config3>(layer2_out, layer3_out);

    layer4_t layer4_out[N_LAYER_4];
    #pragma HLS ARRAY_PARTITION variable=layer4_out complete dim=0
    nnet::dense<layer3_t, layer4_t, config4>(layer3_out, layer4_out, w4, b4);

    layer5_t layer5_out[N_LAYER_4];
    #pragma HLS ARRAY_PARTITION variable=layer5_out complete dim=0
    nnet::relu<layer4_t, layer5_t, relu_config5>(layer4_out, layer5_out);

    layer6_t layer6_out[N_LAYER_6];
    #pragma HLS ARRAY_PARTITION variable=layer6_out complete dim=0
    nnet::dense<layer5_t, layer6_t, config6>(layer5_out, layer6_out, w6, b6);

    result_t layer7_out[N_LAYER_6];
    nnet::softmax<layer6_t, result_t, softmax_config7>(layer6_out, layer7_out);

    //out_class = 0;
    result_t layerN_tmp=0;
    for(int i = 0; i < N_LAYER_6; i++) {
    	//layerN_out[i] = uint32(layer7_out[i].to_float()*65536);

    	axis_out out;
    	out.last = 0;
    	out.data = layer7_out[i].to_float();
		if(i == 1){
			out.last = 1;}
		*layerN_out++ = out;

    	//if(layer7_out[i]>layerN_tmp){
    	//	layerN_tmp = layer7_out[i];
    	//	out_class = i;
    	//}
    }



/*
    for(int i=0;i<N_INPUT_1_1;i++){
        	layer1_out[i] = uint32(input1[i].to_float()*65536);
    }

    for(int i=0;i<N_INPUT_1_1;i++){
    	layer1_out[i].data = uint32(input1[i].to_float()*65536);
    	layer1_out[i].dest = layer1_in[i].dest;
    	layer1_out[i].id = layer1_in[i].id;
    	layer1_out[i].keep = layer1_in[i].keep;
    	layer1_out[i].last = layer1_in[i].last;
    	layer1_out[i].strb = layer1_in[i].strb;
    	layer1_out[i].user = layer1_in[i].user;

    }
*/
}
