#ifndef PARAMETERS_H_
#define PARAMETERS_H_

#include <complex>
#include "ap_int.h"
#include "ap_fixed.h"
#include "nnet_dense.h"
#include "nnet_conv.h"
#include "nnet_conv2d.h"
#include "nnet_activation.h"
#include "nnet_common.h"
#include "nnet_batchnorm.h"
#include "nnet_pooling.h"
#include "nnet_merge.h"

#include <ap_axi_sdata.h>
#include "hls_stream.h"


typedef float float32;
typedef unsigned int uint32;

//hls-fpga-machine-learning insert numbers
#define N_INPUT_1_1 13
#define N_LAYER_2 10
#define N_LAYER_4 10
#define N_LAYER_6 2

//hls-fpga-machine-learning insert layer-precision
typedef ap_fixed<12,4>model_default_t;

typedef ap_fixed<12,4>layer2_t;
typedef ap_fixed<12,4>layer3_t;
typedef ap_fixed<12,4>layer4_t;
typedef ap_fixed<12,4>layer5_t;
typedef ap_fixed<12,4>layer6_t;

//typedef ap_fixed<12,4>input_t;
typedef ap_fixed<12,4>input_t;
typedef ap_fixed<12,4>result_t;

#define N_SHIFT  16

typedef int stream_type;
//typedef ap_axiu<32,1,1,1> stream_type;
//typedef hls::stream<unsigned int> stream_type;

// stream related
typedef union {
	unsigned int as_uint32;
	//stream_type as_stream;
	float as_float32;
} conversion_streamtype;


struct axis_in {
	float data;
//	ap_fixed<16,2> data;
	ap_uint<1> last;
};

struct axis_out {
	float data;
//	ap_int<20> data;
	ap_uint<1> last;
};



//hls-fpga-machine-learning insert layer-config
struct config2 : nnet::dense_config {
    static const unsigned n_in = N_INPUT_1_1;
    static const unsigned n_out = N_LAYER_2;
    static const unsigned io_type = nnet::io_parallel;
    static const unsigned reuse_factor = 1;
    static const unsigned n_zeros = 0;
    static const bool store_weights_in_bram = false;
    typedef model_default_t accum_t;
    typedef model_default_t bias_t;
    typedef model_default_t weight_t;
};

struct relu_config3 : nnet::activ_config {
    static const unsigned n_in = N_LAYER_2;
    static const unsigned table_size = 1024;
    static const unsigned io_type = nnet::io_parallel;
};

struct config4 : nnet::dense_config {
    static const unsigned n_in = N_LAYER_2;
    static const unsigned n_out = N_LAYER_4;
    static const unsigned io_type = nnet::io_parallel;
    static const unsigned reuse_factor = 1;
    static const unsigned n_zeros = 0;
    static const bool store_weights_in_bram = false;
    typedef model_default_t accum_t;
    typedef model_default_t bias_t;
    typedef model_default_t weight_t;
};

struct relu_config5 : nnet::activ_config {
    static const unsigned n_in = N_LAYER_4;
    static const unsigned table_size = 1024;
    static const unsigned io_type = nnet::io_parallel;
};

struct config6 : nnet::dense_config {
    static const unsigned n_in = N_LAYER_4;
    static const unsigned n_out = N_LAYER_6;
    static const unsigned io_type = nnet::io_parallel;
    static const unsigned reuse_factor = 1;
    static const unsigned n_zeros = 0;
    static const bool store_weights_in_bram = false;
    typedef model_default_t accum_t;
    typedef model_default_t bias_t;
    typedef model_default_t weight_t;
};

struct softmax_config7 : nnet::activ_config {
    static const unsigned n_in = N_LAYER_6;
    static const unsigned table_size = 1024;
    static const unsigned io_type = nnet::io_parallel;
};


#endif 
